#include <Arduino.h>
#include "nvd.h"
#include "util.h"
#include "FlashAsEEPROM.h"

// saving and retrieving non-volatile data (NVD) to / from
// ESP8266 flash. 

NVD nvd;

void nvd_Init(void)   {
//	EEPROM.begin(NVD_SIZE_BYTES);
  if (!EEPROM.isValid()) {

	for (int inx = 0; inx < NVD_SIZE_BYTES; inx++) {
		nvd.buf[inx] = EEPROM.read(inx);
		}
	uint16_t checkSum = nvd_CheckSum();	
	//Serial.printf("Calculated checkSum = 0x%04x\r\n", checkSum);	
	//Serial.printf("Saved checkSum = 0x%04x\r\n", nvd.params.checkSum);
	
    if ((checkSum ^ nvd.params.checkSum) == 0xFFFF) {
		Serial.println("nvd checkSum ok");
		char tmpbuffer[50];
		sprintf(tmpbuffer,"axBias = %d", nvd.params.axBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"ayBias = %d", nvd.params.ayBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"azBias = %d", nvd.params.azBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"gxBias = %d", nvd.params.gxBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"gyBias = %d", nvd.params.gyBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"gzBias = %d", nvd.params.gzBias);
		Serial.println(tmpbuffer);
		sprintf(tmpbuffer,"calibrated = 0x%X", nvd.params.calibrated);
		Serial.println(tmpbuffer);
		}
   else  {
		Serial.println("nvd bad checkSum, saving safe values to flash");
		nvd.params.axBias = 0;
		nvd.params.ayBias = 0;
		nvd.params.azBias = 0;
		nvd.params.gxBias = 0;
		nvd.params.gyBias = 0;
		nvd.params.gzBias = 0;
		nvd.params.calibrated = 0;
		checkSum = nvd_CheckSum();
		nvd.params.checkSum = ~checkSum;
	    for  (int inx = 0; inx < NVD_SIZE_BYTES; inx++){
			EEPROM.write(inx, nvd.buf[inx]);
			}
		EEPROM.commit();
		}
//   EEPROM.end();
   
  } else {

    Serial.println("EEPROM has been written.");
    Serial.println("Here is the content of the first 20 bytes:");

    Serial.print("->");
    for (int i=0; i<20; i++) {
      Serial.print(" ");
      Serial.print(EEPROM.read(i));
    }
    Serial.println();

  }
   
}

   
uint16_t nvd_CheckSum(void) {
	uint16_t checkSum = 0;
	for (int inx = 0; inx < NVD_SIZE_BYTES-2; inx++) {
		checkSum += (uint16_t)nvd.buf[inx]; 
		}
	return checkSum;
	}
	

void nvd_SaveCalibrationParams(int16_t axb, int16_t ayb, int16_t azb, int16_t gxb, int16_t gyb, int16_t gzb, int16_t calibrated) {
//	EEPROM.begin(NVD_SIZE_BYTES);
	nvd.params.axBias = axb;
	nvd.params.ayBias = ayb;
	nvd.params.azBias = azb;
	nvd.params.gxBias = gxb;
	nvd.params.gyBias = gyb;
	nvd.params.gzBias = gzb;
	nvd.params.calibrated = calibrated;
	uint16_t checkSum = nvd_CheckSum();
	nvd.params.checkSum = ~checkSum;
	for  (int inx = 0; inx < NVD_SIZE_BYTES; inx++){
		EEPROM.write(inx, nvd.buf[inx]);
		}
	EEPROM.commit();

	Serial.print("After commit, calling isValid() returns ");
    Serial.println(EEPROM.isValid());

    Serial.println("EEPROM has been written.");
    Serial.println("Here is the content of the first 20 bytes:");

    Serial.print("->");
    for (int i=0; i<20; i++) {
      Serial.print(" ");
      Serial.print(EEPROM.read(i));
    }
    Serial.println();
}	


